//+--------------------------------------------------------------------------+
//|                                                                          |
//|                                  MD2ex.h                                 |
//|                                                                          |
//|           C++ MD2 API encapsulation constants and declarations.          |
//|                                                                          |
//|                               Version 1.00                               |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|            8 Domaine de Rochagnon. 38800 Champagnier  FRANCE             |
//|                       http://www.zapsolution.com                         |
//|                    E-mail: pterrier@zapsolution.com                      |
//|                                                                          |
//|                   copyright (c) 2010 Patrice TERRIER                     |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|          C++ translation started on : 08-03-2010                         |
//|                        Last revised : 08-08-2010                         |
//+--------------------------------------------------------------------------+

#include <windows.h>
//
#include <iostream>
#include <string.h>

// Macro definitions.
#define int_proc typedef int (__stdcall *zProc)
#define string_proc typedef char* (__stdcall *zProc)
#define void_proc typedef void (__stdcall *zProc)
#define handle_proc typedef HANDLE (__stdcall *zProc)

typedef struct {
    int ID;
    char szFullName[260];
    int nTexture;
    int nSquare;            // boolean TRUE or FALSE value
    int nWidth;
    int nHeight;
} ZGLTEXTURE_EX;

// Load the MD2ex module.
HMODULE MD2ex () {
    static HMODULE hLib;
    if (!hLib) { hLib = LoadLibraryA ("MD2ex"); }
    return hLib;
}

//DECLARE FUNCTION MD2_Load LIB "MD2ex.DLL" ALIAS "MD2_Load" (BYVAL index AS LONG, BYVAL s AS STRING, BYVAL n AS LONG, BYVAL w AS LONG, BYVAL h AS LONG) AS LONG
int MD2_Load (
    int index,
    std::string sFullPath,
    int nTexture,
    int TextureWidth,
    int TextureHeight
    ) {

    int nRet = 0;

    if (MD2ex()) {
        int_proc (int, char*, int, int, int);
        zProc hPROC = (zProc) GetProcAddress(MD2ex(), "MD2_Load");
        if (hPROC) {
            nRet = hPROC(index, (char*) sFullPath.c_str(), nTexture, TextureWidth, TextureHeight);
        }
    }
    return nRet;
}

int MD2_GetFrameCount (
    int index
    ) {

    int nCount = 0;

    if (MD2ex()) {
        int_proc (int);
        zProc hPROC = (zProc) GetProcAddress(MD2ex(), "MD2_GetFrameCount");
        if (hPROC) {
            nCount = hPROC(index);
        }
    }
    return nCount;
}

int MD2_Draw (
    int index,
    float fFrame
    ) {

    int nRet = 0;

    if (MD2ex()) {
        int_proc (int, float);
        zProc hPROC = (zProc) GetProcAddress(MD2ex(), "MD2_Draw");
        if (hPROC) {
            nRet = hPROC(index, fFrame);
        }
    }
    return nRet;
}

int MD2_Delete (
    int index
    ) {

    int nRet = 0;

    if (MD2ex()) {
        int_proc (int);
        zProc hPROC = (zProc) GetProcAddress(MD2ex(), "MD2_Delete");
        if (hPROC) {
           nRet = hPROC(index);
        }
    }
    return nRet;
}

int MD2_MakeMultipleTexture (
    ZGLTEXTURE_EX TextureArray[],
    int nCount
    ) {

    int nRet = 0;

    if (MD2ex()) {
        int_proc (ZGLTEXTURE_EX*, int);
        zProc hPROC = (zProc) GetProcAddress(MD2ex(), "MD2_MakeMultipleTexture");
        if (hPROC) {
           nRet = hPROC(&TextureArray[0], nCount);
        }
    }
    return nRet;
}

void MD2_DestroyTexture (
    ZGLTEXTURE_EX TextureArray[],
    int nCount
    ) {

    if (MD2ex()) {
        void_proc (ZGLTEXTURE_EX*, int);
        zProc hPROC = (zProc) GetProcAddress(MD2ex(), "MD2_DestroyTexture");
        if (hPROC) {
           hPROC(&TextureArray[0], nCount);
        }
    }
}

int MD2_DrawAnim (
    int index,
    float fFrame
    ) {

    int nRet = 0;

    if (MD2ex()) {
        int_proc (int, float);
        zProc hPROC = (zProc) GetProcAddress(MD2ex(), "MD2_DrawAnim");
        if (hPROC) {
            nRet = hPROC(index, fFrame);
        }
    }
    return nRet;
}

int MD2_LockAnimRangeByName (
    int index,
    int lBound,
    int uBound
    ) {

    int nRet = 0;

    if (MD2ex()) {
        int_proc (int, int, int);
        zProc hPROC = (zProc) GetProcAddress(MD2ex(), "MD2_LockAnimRangeByName");
        if (hPROC) {
            nRet = hPROC(index, lBound, uBound);
        }
    }
    return nRet;
}

int MD2_AdvanceAnim (
    int index,
    float fFrame
    ) {

    int nRet = 0;

    if (MD2ex()) {
        int_proc (int, float);
        zProc hPROC = (zProc) GetProcAddress(MD2ex(), "MD2_AdvanceAnim");
        if (hPROC) {
            nRet = hPROC(index, fFrame);
        }
    }
    return nRet;
}

int MD2_LockAnimRangeByName (
    int index,
    std::string sName
    ) {

    int nRet = 0;

    if (MD2ex()) {
        int_proc (int, char*);
        zProc hPROC = (zProc) GetProcAddress(MD2ex(), "MD2_LockAnimRangeByName");
        if (hPROC) {
           nRet = hPROC(index, (char*) sName.c_str());
        }
    }
    return nRet;
}

